@@ -24,6 +24,7 @@ use CGI::Carp qw(fatalsToBrowser carpout);
 require "ow-shared.pl";
 require "filelock.pl";
 
+# common globals
 use vars qw(%config %config_raw);
 use vars qw($thissession);
 use vars qw($domain $user $userrealname $uuid $ugid $homedir);
@@ -31,20 +32,25 @@ use vars qw(%prefs %style %icontext);
 use vars qw($folderdir @validfolders $folderusage);
 use vars qw($folder $printfolder $escapedfolder);
 
-openwebmail_init();
+# extern vars
+use vars qw(%lang_folders %lang_sizes %lang_text %lang_err);	# defined in lang/xy
 
+# local globals
 use vars qw($sort $page);
 use vars qw($messageid $escapedmessageid);
 
+########################## MAIN ##############################
+openwebmail_requestbegin();
+$SIG{PIPE}=\&openwebmail_exit;	# for user stop
+$SIG{TERM}=\&openwebmail_exit;	# for user stop
+
+userenv_init();
+
 $page = param("page") || 1;
 $sort = param("sort") || $prefs{'sort'} || 'date';
 $messageid=param("message_id") || '';
 $escapedmessageid=escapeURL($messageid);
 
-# extern vars
-use vars qw(%lang_folders %lang_sizes %lang_text %lang_err);	# defined in lang/xy
-
-########################## MAIN ##############################
 my $action = param("action");
 if ($action eq "addressbook") {
    addressbook();
@@ -63,11 +69,10 @@ if ($action eq "addressbook") {
 } elsif ($action eq "exportabook") {
    exportabook();
 } else {
-   openwebmailerror("Action $lang_err{'has_illegal_chars'}");
+   openwebmailerror(__FILE__, __LINE__, "Action $lang_err{'has_illegal_chars'}");
 }
 
-# back to root if possible, required for setuid under persistent perl
-$<=0; $>=0;
+openwebmail_requestend();
 ###################### END MAIN ##############################
 
 #################### ADDRESSBOOK #######################
@@ -90,8 +95,7 @@ sub addressbook {
    my $results_flag = 0;
 
    my ($html, $temphtml);
-   $html = readtemplate("addressbook.template");
-   $html = applystyle($html);
+   $html = applystyle(readtemplate("addressbook.template"));
 
    if (defined($lang_text{$field})) {
       $temphtml=$lang_text{$field}.": $lang_text{'abook'}";
@@ -340,7 +344,7 @@ sub addressbook {
    $html =~ s/\@\@\@FORMNAME\@\@\@/$form/g;
    $html =~ s/\@\@\@FIELDNAME\@\@\@/$field/g;
 
-   print htmlheader(), $html, htmlfooter();
+   httpprint([], [htmlheader(), $html, htmlfooter(0)]);
 }
 ################## END ADDRESSBOOK #####################
 
@@ -359,7 +363,7 @@ sub importabook {
       close($abookupload);
 #      if ($mua eq 'outlookexp5') {
 #         unless ($abookcontents =~ /^Name,E-mail Address/) {
-#            openwebmailerror(qq|$lang_err{'abook_invalid'} <a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=importabook&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page&amp;message_id=$escapedmessageid">$lang_err{'back'}</a> $lang_err{'tryagain'}|);
+#            openwebmailerror(__FILE__, __LINE__, qq|$lang_err{'abook_invalid'} <a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=importabook&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page&amp;message_id=$escapedmessageid">$lang_err{'back'}</a> $lang_err{'tryagain'}|);
 #         }
 #      }
       if (! -f "$folderdir/.address.book" ) {
@@ -367,9 +371,9 @@ sub importabook {
          close(ABOOK);
       }
       filelock("$folderdir/.address.book", LOCK_EX|LOCK_NB) or
-         openwebmailerror("$lang_err{'couldnt_lock'} $folderdir/.address.book!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdir/.address.book!");
       open (ABOOK,"+<$folderdir/.address.book") or
-         openwebmailerror("$lang_err{'couldnt_open'} $folderdir/.address.book!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderdir/.address.book! ($!)");
       while (<ABOOK>) {
          ($name, $email, $note) = split(/\@\@\@/, $_, 3);
          chomp($email); chomp($note);
@@ -379,36 +383,36 @@ sub importabook {
       foreach my $line (split(/\r*\n/, $abookcontents)) {
  #        next if ( ($mua eq 'outlookexp5') && (/^Name,E-mail Address/) );
          next if ( $line !~ /\@/ );
-         my @fields = str2list($line,1);
-         if ( ($mua eq 'outlookexp5') && ($fields[0]) && ($fields[1]) ) {
-            $fields[0] =~ s/://;
-            $fields[0] =~ s/</&lt;/g;
-            $fields[0] =~ s/>/&gt;/g;
-            $fields[1] =~ s/</&lt;/g;
-            $fields[1] =~ s/>/&gt;/g;
-            $addresses{$fields[0]} = $fields[1];
-            $note = join(",", @fields[2..9]);
-            $note =~ s/,\s*,//g;
-            $note =~ s/^\s*,\s*//g;
-            $note =~ s/\s*,\s*$//g;
-            $notes{$fields[0]} = $note;
-         } elsif ( ($mua eq 'nsmail') && ($fields[0]) && ($fields[6]) ) {
-            $fields[0] =~ s/://;
-            $fields[0] =~ s/</&lt;/g;
-            $fields[0] =~ s/>/&gt;/g;
-            $fields[6] =~ s/</&lt;/g;
-            $fields[6] =~ s/>/&gt;/g;
-            $addresses{"$fields[0]"} = $fields[6];
-            $note = join(",", @fields[1..5,7..9]);
-            $note =~ s/,\s*,//g;
-            $note =~ s/^\s*,\s*//g;
-            $note =~ s/\s*,\s*$//g;
-            $notes{$fields[0]} = $note;
+         my @t = str2list($line,1);
+         if ( $mua eq 'outlookexp5' && $t[0] && $t[1] ) {
+            $name=shift(@t);
+            $name =~ s/</&lt;/g; $name =~ s/>/&gt;/g;
+            $email=shift(@t);
+            while ($t[0]=~/^[\w\d\.-_]+\@[\w\d-_]+(\.[\w\d-_]+)+$/) {
+               $email.=",".shift(@t);	# for owm group addr import
+            }
+            $email =~ s/</&lt;/g; $email =~ s/>/&gt;/g;
+            $note = join(",", @t);
+            $note =~ s/,\s*,//g; $note =~ s/^\s*,\s*//g; $note =~ s/\s*,\s*$//g;
+
+            $addresses{$name} = $email;
+            $notes{$name} = $note;
+
+         } elsif ( $mua eq 'nsmail' && $t[0] && $t[6] ) {
+            $name=$t[0];
+            $name =~ s/</&lt;/g; $name =~ s/>/&gt;/g;
+            $email=$t[6];
+            $email =~ s/</&lt;/g; $email =~ s/>/&gt;/g;
+            $note = join(",", @t[1..5,7..9]);
+            $note =~ s/,\s*,//g; $note =~ s/^\s*,\s*//g; $note =~ s/\s*,\s*$//g;
+
+            $addresses{$name} = $email;
+            $notes{$name} = $note;
          }
       }
 
       seek (ABOOK, 0, 0) or
-         openwebmailerror("$lang_err{'couldnt_seek'} $folderdir/.address.book!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_seek'} $folderdir/.address.book! ($!)");
 
       foreach (sort keys %addresses) {
          ($name,$email,$note)=($_, $addresses{$_}, $notes{$_});
@@ -418,20 +422,19 @@ sub importabook {
       }
 
       if (length($abooktowrite) > ($config{'maxbooksize'} * 1024)) {
-         openwebmailerror(qq|$lang_err{'abook_toobig'}|.
+         openwebmailerror(__FILE__, __LINE__, qq|$lang_err{'abook_toobig'}|.
                           qq|<a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=importabook&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page&amp;message_id=$escapedmessageid">$lang_err{'back'}</a>|.
                           qq|$lang_err{'tryagain'}|);
       }
       print ABOOK $abooktowrite;
       truncate(ABOOK, tell(ABOOK));
 
-      close (ABOOK) or openwebmailerror("$lang_err{'couldnt_close'} $folderdir/.address.book!");
+      close (ABOOK) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} $folderdir/.address.book! ($!)");
       filelock("$folderdir/.address.book", LOCK_UN);
 
-      writelog("abok - import addressbook");
-      writehistory("abook - import addressbook");
+      writelog("import addressbook");
+      writehistory("import addressbook");
 
-#      print "Location: $config{'ow_cgiurl'}/openwebmail-abook.pl?action=editaddresses&sessionid=$thissession&sort=$sort&folder=$escapedfolder&page=$page&message_id=$escapedmessageid\n\n";
       editaddresses();
 
    } else {
@@ -440,8 +443,7 @@ sub importabook {
       my $freespace = int($config{'maxbooksize'} - ($abooksize/1024) + .5);
 
       my ($html, $temphtml);
-      $html = readtemplate("importabook.template");
-      $html = applystyle($html);
+      $html = applystyle(readtemplate("importabook.template"));
 
       $html =~ s/\@\@\@FREESPACE\@\@\@/$freespace $lang_sizes{'kb'}/g;
 
@@ -511,7 +513,7 @@ sub importabook {
       $temphtml = submit("$lang_text{'cancel'}");
       $html =~ s/\@\@\@CANCELBUTTON\@\@\@/$temphtml/;
 
-      print htmlheader(), $html, htmlfooter(2);
+      httpprint([], [htmlheader(), $html, htmlfooter(2)]);
    }
 }
 
@@ -527,9 +529,9 @@ sub importabook_pine {
       my $abooktowrite='';
 
       filelock("$folderdir/.address.book", LOCK_EX|LOCK_NB) or
-         openwebmailerror("$lang_err{'couldnt_lock'} $folderdir/.address.book!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdir/.address.book!");
       open (ABOOK,"+<$folderdir/.address.book") or
-         openwebmailerror("$lang_err{'couldnt_open'} $folderdir/.address.book!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderdir/.address.book! ($!)");
 
       while (<ABOOK>) {
          ($name, $email, $note) = split(/\@\@\@/, $_, 3);
@@ -549,7 +551,7 @@ sub importabook_pine {
       close (PINEBOOK);
 
       seek (ABOOK, 0, 0) or
-         openwebmailerror("$lang_err{'couldnt_seek'} $folderdir/.address.book!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_seek'} $folderdir/.address.book! ($!)");
 
       foreach (sort keys %addresses) {
          ($name,$email,$note)=($_, $addresses{$_}, $notes{$_});
@@ -559,18 +561,18 @@ sub importabook_pine {
       }
 
       if (length($abooktowrite) > ($config{'maxbooksize'} * 1024)) {
-         openwebmailerror(qq|$lang_err{'abook_toobig'}|.
+         openwebmailerror(__FILE__, __LINE__, qq|$lang_err{'abook_toobig'}|.
                           qq|<a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=importabook&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page&amp;message_id=$escapedmessageid">$lang_err{'back'}</a>|.
                           qq|$lang_err{'tryagain'}|);
       }
       print ABOOK $abooktowrite;
       truncate(ABOOK, tell(ABOOK));
 
-      close (ABOOK) or openwebmailerror("$lang_err{'couldnt_close'} $folderdir/.address.book!");
+      close (ABOOK) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} $folderdir/.address.book! ($!)");
       filelock("$folderdir/.address.book", LOCK_UN);
 
-      writelog("abook - import addressbook");
-      writehistory("abook - import addressbook");
+      writelog("import addressbook");
+      writehistory("import addressbook");
    }
    editaddresses();
 }
@@ -579,15 +581,16 @@ sub importabook_pine {
 ##################### EXPORTABOOK ############################
 sub exportabook {
    filelock("$folderdir/.address.book", LOCK_EX|LOCK_NB) or
-      openwebmailerror("$lang_err{'couldnt_lock'} $folderdir/.address.book!");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdir/.address.book!");
    open (ABOOK,"$folderdir/.address.book") or
-      openwebmailerror("$lang_err{'couldnt_open'} $folderdir/.address.book!");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderdir/.address.book! ($!)");
 
    # disposition:attachment default to save
-   print qq|Content-Transfer-Coding: binary\n|,
-         qq|Connection: close\n|,
+   print qq|Connection: close\n|,
          qq|Content-Type: text/plain; name="adbook.csv"\n|;
-   if ( $ENV{'HTTP_USER_AGENT'}!~/MSIE 5.5/ ) {	# ie5.5 is broken with content-disposition
+   if ( $ENV{'HTTP_USER_AGENT'}=~/MSIE 5.5/ ) {	# ie5.5 is broken with content-disposition: attachment
+      print qq|Content-Disposition: filename="adbook.csv"\n|;
+   } else {
       print qq|Content-Disposition: attachment; filename="adbook.csv"\n|;
    }
    print qq|\n|;
@@ -599,8 +602,8 @@ sub exportabook {
    close(ABOOK);
    filelock("$folderdir/.address.book", LOCK_UN);
 
-   writelog("abook - export addressbook");
-   writehistory("abook - export addressbook");
+   writelog("export addressbook");
+   writehistory("export addressbook");
 
    return;
 }
@@ -618,19 +621,18 @@ sub editaddresses {
    my $abook_searchtype = param("abook_searchtype") || 'name';
 
    my ($html, $temphtml);
-   $html = readtemplate("editaddresses.template");
-   $html = applystyle($html);
+   $html = applystyle(readtemplate("editaddresses.template"));
 
    if ( -f "$folderdir/.address.book" ) {
       open (ABOOK,"$folderdir/.address.book") or
-         openwebmailerror("$lang_err{'couldnt_open'} $folderdir/.address.book!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderdir/.address.book! ($!)");
       while (<ABOOK>) {
          ($name, $email, $note) = split(/\@\@\@/, $_, 3);
          chomp($email); chomp($note);
          $addresses{"$name"} = $email;
          $notes{"$name"}=$note;
       }
-      close (ABOOK) or openwebmailerror("$lang_err{'couldnt_close'} $folderdir/.address.book!");
+      close (ABOOK) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} $folderdir/.address.book! ($!)");
    }
    my $abooksize = ( -s "$folderdir/.address.book" ) || 0;
    my $freespace = int($config{'maxbooksize'} - ($abooksize/1024) + .5);
@@ -879,7 +881,7 @@ sub editaddresses {
    }
    $html =~ s/\@\@\@ADDRESSES\@\@\@/$temphtml/;
 
-   print htmlheader(), $html, htmlfooter(2);
+   httpprint([], [htmlheader(), $html, htmlfooter(2)]);
 }
 ################### END EDITADDRESSES ########################
 
@@ -903,13 +905,13 @@ sub modaddress {
       if ( -f "$folderdir/.address.book" ) {
          if ($mode ne 'delete') {
             if ( (-s "$folderdir/.address.book") >= ($config{'maxbooksize'} * 1024) ) {
-               openwebmailerror(qq|$lang_err{'abook_toobig'} <a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=editaddresses&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page&amp;message_id=$escapedmessageid">$lang_err{'back'}</a>$lang_err{'tryagain'}|);
+               openwebmailerror(__FILE__, __LINE__, qq|$lang_err{'abook_toobig'} <a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=editaddresses&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page&amp;message_id=$escapedmessageid">$lang_err{'back'}</a>$lang_err{'tryagain'}|);
             }
          }
          filelock("$folderdir/.address.book", LOCK_EX|LOCK_NB) or
-            openwebmailerror("$lang_err{'couldnt_lock'} $folderdir/.address.book!");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdir/.address.book!");
          open (ABOOK,"+<$folderdir/.address.book") or
-            openwebmailerror("$lang_err{'couldnt_open'} $folderdir/.address.book!");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderdir/.address.book! ($!)");
          while (<ABOOK>) {
             ($name, $email, $note) = split(/\@\@\@/, $_, 3);
             chomp($email); chomp($note);
@@ -927,7 +929,7 @@ sub modaddress {
             }
          }
          seek (ABOOK, 0, 0) or
-            openwebmailerror("$lang_err{'couldnt_seek'} $folderdir/.address.book!");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_seek'} $folderdir/.address.book! ($!)");
 
          foreach (sort keys %addresses) {
             ($name,$email,$note)=($_, $addresses{$_}, $notes{$_});
@@ -936,15 +938,15 @@ sub modaddress {
             print ABOOK "$name\@\@\@$email\@\@\@$note\n";
          }
          truncate(ABOOK, tell(ABOOK));
-         close (ABOOK) or openwebmailerror("$lang_err{'couldnt_close'} $folderdir/.address.book!");
+         close (ABOOK) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} $folderdir/.address.book! ($!)");
          filelock("$folderdir/.address.book", LOCK_UN);
       } else {
          open (ABOOK, ">$folderdir/.address.book" ) or
-            openwebmailerror("$lang_err{'couldnt_open'} $folderdir/.address.book!");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderdir/.address.book! ($!)");
          $realname=~s/\@\@/\@\@ /g; $realname=~s/\@$/\@ /;
          $address=~s/\@\@/\@\@ /g; $address=~s/\@$/\@ /;
          print ABOOK "$realname\@\@\@$address\@\@\@$usernote\n";
-         close (ABOOK) or openwebmailerror("$lang_err{'couldnt_close'} $folderdir/.address.book!");
+         close (ABOOK) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} $folderdir/.address.book! ($!)");
       }
    }
 
@@ -952,7 +954,7 @@ sub modaddress {
       my $searchtype = param("searchtype") || 'subject';
       my $keyword = param("keyword") || '';
       my $escapedkeyword = escapeURL($keyword);
-      print "Location: $config{'ow_cgiurl'}/openwebmail-read.pl?action=readmessage&sessionid=$thissession&page=$page&sort=$sort&keyword=$escapedkeyword&searchtype=$searchtype&folder=$escapedfolder&message_id=$escapedmessageid\n\n";
+      print redirect(-location=>"$config{'ow_cgiurl'}/openwebmail-read.pl?sessionid=$thissession&folder=$escapedfolder&page=$page&sort=$sort&keyword=$escapedkeyword&searchtype=$searchtype&message_id=$escapedmessageid&action=readmessage&headers=$prefs{'headers'}&attmode=simple");
    } else {
       editaddresses();
    }
@@ -963,14 +965,13 @@ sub modaddress {
 sub clearaddress {
    if ( -f "$folderdir/.address.book" ) {
       open (ABOOK, ">$folderdir/.address.book") or
-         openwebmailerror ("$lang_err{'couldnt_open'} $folderdir/.address.book!");
-      close (ABOOK) or openwebmailerror("$lang_err{'couldnt_close'} $folderdir/.address.book!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderdir/.address.book! ($!)");
+      close (ABOOK) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} $folderdir/.address.book! ($!)");
    }
 
-   writelog("abook - clear addressbook");
-   writehistory("abook - clear addressbook");
+   writelog("clear addressbook");
+   writehistory("clear addressbook");
 
-#   print "Location: $config{'ow_cgiurl'}/openwebmail-abook.pl?action=editaddresses&sessionid=$thissession&sort=$sort&folder=$escapedfolder&page=$page&message_id=$escapedmessageid\n\n";
    editaddresses();
 }
 ################## END CLEARADDRESS ###########################
