@@ -25,26 +25,34 @@ require "ow-shared.pl";
 require "filelock.pl";
 require "mime.pl";
 require "maildb.pl";
+require "htmltext.pl";
 
+# common globals
 use vars qw(%config %config_raw);
 use vars qw($thissession);
 use vars qw($domain $user $userrealname $uuid $ugid $homedir);
 use vars qw(%prefs %style %icontext);
+use vars qw($quotausage $quotalimit);
 use vars qw($folderdir @validfolders $folderusage);
 use vars qw($folder $printfolder $escapedfolder);
 
-openwebmail_init();
-
-use vars qw($sort $page);
-
-$page = param("page") || 1;
-$sort = param("sort") || $prefs{'sort'} || 'date';
-
 # extern vars
 use vars qw(%lang_folders %lang_text %lang_err);	# defined in lang/xy
 use vars qw($_OFFSET $_STATUS);				# defined in maildb.pl
 
+# local globals
+use vars qw($sort $page);
+
 ########################## MAIN ##############################
+openwebmail_requestbegin();
+$SIG{PIPE}=\&openwebmail_exit;	# for user stop
+$SIG{TERM}=\&openwebmail_exit;	# for user stop
+
+userenv_init();
+
+$page = param("page") || 1;
+$sort = param("sort") || $prefs{'sort'} || 'date';
+
 my $action = param("action");
 if ($action eq "editfolders") {
    editfolders();
@@ -65,11 +73,10 @@ if ($action eq "editfolders") {
 } elsif ($action eq "downloadfolder") {
    downloadfolder();
 } else {
-   openwebmailerror("Action $lang_err{'has_illegal_chars'}");
+   openwebmailerror(__FILE__, __LINE__, "Action $lang_err{'has_illegal_chars'}");
 }
 
-# back to root if possible, required for setuid under persistent perl
-$<=0; $>=0;
+openwebmail_requestend();
 ###################### END MAIN ##############################
 
 #################### EDITFOLDERS ###########################
@@ -96,8 +103,7 @@ sub editfolders {
    }
 
    my ($html, $temphtml);
-   $html = readtemplate("editfolders.template");
-   $html = applystyle($html);
+   $html = applystyle(readtemplate("editfolders.template"));
 
    $html =~ s/\@\@\@FOLDERNAME_MAXLEN\@\@\@/$config{'foldername_maxlen'}/g;
 
@@ -171,16 +177,22 @@ sub editfolders {
    $html =~ s/\@\@\@DEFAULTFOLDERS\@\@\@/$temphtml/;
 
    my $usagestr;
-   if ($config{'folderquota'}) {
-      if ($folderusage>=90) {
-         $usagestr="<B><font color='#cc0000'>$folderusage %</font></B>";
+   if ($config{'quota_module'} ne "none") {
+      my $percent=0;
+      $usagestr="$lang_text{'quotausage'}: ".lenstr($quotausage*1024,1);
+      if ($quotalimit>0) {
+         $percent=int($quotausage*1000/$quotalimit)/10;
+         $usagestr.=" ($percent%) &nbsp;";
+         $usagestr.="$lang_text{'quotalimit'}: ".lenstr($quotalimit*1024,1);
+      }         
+      if ($percent>=90) {
+         $usagestr="<B><font color='#cc0000'>$usagestr</font></B>";
       } else {
-         $usagestr="<B>$folderusage %</B>";
+         $usagestr="<B>$usagestr</B>";
       }
    } else {
       $usagestr="&nbsp;";
    }
-
    $total_foldersize=lenstr($total_foldersize,0);
 
    $temphtml = qq|<tr>|.
@@ -192,7 +204,7 @@ sub editfolders {
                qq|</tr>|;
    $html =~ s/\@\@\@TOTAL\@\@\@/$temphtml/;
 
-   print htmlheader(), $html, htmlfooter(1);
+   httpprint([], [htmlheader(), $html, htmlfooter(1)]);
 }
 
 # this is inline function used by sub editfolders(), it changes
@@ -207,7 +219,7 @@ sub _folderline {
    if ( -f "$headerdb$config{'dbm_ext'}" && !-z "$headerdb$config{'dbm_ext'}" ) {
       if (!$config{'dbmopen_haslock'}) {
          filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) or
-            openwebmailerror("$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
       }
       dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
       if ( defined($HDB{'ALLMESSAGES'}) ) {
@@ -237,8 +249,7 @@ sub _folderline {
 
    my $escapedcurrfolder = escapeURL($currfolder);
    my $url = "$config{'ow_cgiurl'}/openwebmail-folder.pl?sessionid=$thissession&amp;folder=$escapedcurrfolder&amp;action=downloadfolder";
-   my $folderstr=$currfolder;
-   $folderstr=$lang_folders{$currfolder} if defined($lang_folders{$currfolder});
+   my $folderstr=$lang_folders{$currfolder}||$currfolder;
 
    my $accesskeystr=$i%10+1;
    if ($accesskeystr == 10) {
@@ -250,7 +261,7 @@ sub _folderline {
    $temphtml .= qq|<tr>|.
                 qq|<td align="center" bgcolor=$bgcolor>|.
                 qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=listmessages&amp;sessionid=$thissession&amp;sort=$sort&amp;page=$page&amp;folder=$escapedcurrfolder">|.
-                qq|$folderstr </a>&nbsp;\n|.
+                str2html($folderstr).qq| </a>&nbsp;\n|.
                 iconlink("download.gif", "$lang_text{'download'} $folderstr ", qq|$accesskeystr href="$url"|).
                 qq|</td>\n|.
                 qq|<td align="center" bgcolor=$bgcolor>$newmessages</td>|.
@@ -319,7 +330,7 @@ sub refreshfolders {
       my ($folderfile,$headerdb)=get_folderfile_headerdb($user, $currfolder);
 
       filelock($folderfile, LOCK_EX|LOCK_NB) or
-         openwebmailerror("$lang_err{'couldnt_lock'} $folderfile!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderfile!");
       if (update_headerdb($headerdb, $folderfile)<0) {
          $errcount++;
          writelog("db error - Couldn't update index db $headerdb$config{'dbm_ext'}");
@@ -328,10 +339,14 @@ sub refreshfolders {
       filelock($folderfile, LOCK_UN);
    }
 
-   writelog("refresh folders - $errcount errors");
-   writehistory("refresh folders - $errcount errors");
+   writelog("folder - refresh, $errcount errors");
+   writehistory("folder - refresh, $errcount errors");
 
-   getfolders(\@validfolders, \$folderusage);
+   # get uptodate quota/folder usage info
+   getfolders(\@validfolders, \$folderusage);   
+   if ($config{'quota_module'} ne 'none') {
+      $quotausage=(quota_get_usage_limit(\%config, $user, $homedir, 1))[2];
+   }
    editfolders();
 }
 ################### END REFRESHFOLDERS ##########################
@@ -343,17 +358,17 @@ sub markreadfolder {
    my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertomark);
 
    filelock($folderfile, LOCK_EX|LOCK_NB) or
-      openwebmailerror("$lang_err{'couldnt_lock'} $folderfile!");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderfile!");
 
    if (update_headerdb($headerdb, $folderfile)<0) {
       filelock($folderfile, LOCK_UN);
-      openwebmailerror("$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
    }
 
    my (%HDB, %offset, %status);
    if (!$config{'dbmopen_haslock'}) {
       filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) or
-         openwebmailerror("$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
    }
    dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    foreach my $messageid (keys %HDB) {
@@ -377,12 +392,14 @@ sub markreadfolder {
    ($tmpfile =~ /^(.+)$/) && ($tmpfile = $1);
    ($tmpdb =~ /^(.+)$/) && ($tmpdb = $1);
 
+   open(F, ">$tmpfile"); close(F);
    filelock("$tmpfile", LOCK_EX) or
-      openwebmailerror("$lang_err{'couldnt_lock'} $tmpfile");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $tmpfile");
 
    if (update_headerdb($tmpdb, $tmpfile)<0) {
       filelock($tmpfile, LOCK_UN);
-      openwebmailerror("$lang_err{'couldnt_updatedb'} $tmpdb$config{'dbm_ext'}");
+      unlink("$tmpdb$config{'dbm_ext'}", $tmpfile);
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} $tmpdb$config{'dbm_ext'}");
    }
 
    foreach my $messageid (sort { $offset{$a}<=>$offset{$b} } keys %offset) {
@@ -435,7 +452,7 @@ sub reindexfolder {
       my %HDB;
       if (!$config{'dbmopen_haslock'}) {
          filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) or
-            openwebmailerror("$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
       }
       dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
       $HDB{'METAINFO'}={'RENEW'};
@@ -444,7 +461,7 @@ sub reindexfolder {
    }
    if (update_headerdb($headerdb, $folderfile)<0) {
       filelock($folderfile, LOCK_UN);
-      openwebmailerror("$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
    }
 
    if ($recreate) {
@@ -455,7 +472,7 @@ sub reindexfolder {
       writehistory("chkindex folder - $foldertoindex");
    }
 
-#   print "Location: $config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&page=$page\n\n";
+#   print redirect(-location=>"$config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&page=$page");
    getfolders(\@validfolders, \$folderusage);
    editfolders();
 }
@@ -463,25 +480,32 @@ sub reindexfolder {
 
 ################### ADDFOLDER ##############################
 sub addfolder {
+   if ($quotalimit>0 && $quotausage>$quotalimit) {
+      $quotausage=(quota_get_usage_limit(\%config, $user, $homedir, 1))[2];	# get uptodate quotausage
+      if ($quotausage>$quotalimit) {
+         openwebmailerror(__FILE__, __LINE__, $lang_err{'quotahit_alert'});
+      }
+   }
+
    my $foldertoadd = safefoldername(param('foldername')) || '';
    ($foldertoadd =~ /^(.+)$/) && ($foldertoadd = $1);
 
    if (length($foldertoadd) > $config{'foldername_maxlen'}) {
-      openwebmailerror("$lang_err{'foldername_long'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'foldername_long'}");
    }
    if ( is_defaultfolder($foldertoadd) ||
         $foldertoadd eq "$user" || $foldertoadd eq "" ) {
-      openwebmailerror("$lang_err{'cant_create_folder'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_folder'}");
    }
 
    my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertoadd);
    if ( -f $folderfile ) {
-      openwebmailerror ("$lang_err{'folder_with_name'} $foldertoadd $lang_err{'already_exists'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'folder_with_name'} $foldertoadd $lang_err{'already_exists'}");
    }
 
    open (FOLDERTOADD, ">$folderfile") or
-      openwebmailerror("$lang_err{'cant_create_folder'} $foldertoadd!");
-   close (FOLDERTOADD) or openwebmailerror("$lang_err{'couldnt_close'} $foldertoadd!");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_folder'} $foldertoadd! ($!)");
+   close (FOLDERTOADD) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} $foldertoadd! ($!)");
 
    # create empty index dbm with mode 0600
    my %HDB;
@@ -541,7 +565,11 @@ sub deletefolder {
       writehistory("delete folder - $foldertodel");
    }
 
+   # get uptodate quota/folder usage info
    getfolders(\@validfolders, \$folderusage);
+   if ($quotalimit>0 && $quotausage>$quotalimit) {
+      $quotausage=(quota_get_usage_limit(\%config, $user, $homedir, 1))[2];
+   }
    editfolders();
 }
 ################### END DELETEFOLDER ##########################
@@ -559,14 +587,14 @@ sub renamefolder {
    ($newname =~ /^(.+)$/) && ($newname = $1);
 
    if (length($newname) > $config{'foldername_maxlen'}) {
-      openwebmailerror("$lang_err{'foldername_long'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'foldername_long'}");
    }
    if ( is_defaultfolder($newname) ||
         $newname eq "$user" || $newname eq "" ) {
-      openwebmailerror("$lang_err{'cant_create_folder'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_folder'}");
    }
    if ( -f "$folderdir/$newname" ) {
-      openwebmailerror ("$lang_err{'folder_with_name'} $newname $lang_err{'already_exists'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'folder_with_name'} $newname $lang_err{'already_exists'}");
    }
 
    if ( -f "$folderdir/$oldname" ) {
@@ -582,11 +610,11 @@ sub renamefolder {
       rename("$oldheaderdb.cache",             "$newheaderdb.cache");
       unlink("$folderdir/$newname.lock",       "$folderdir/$oldname.lock.lock");
 
-      writelog("rename folder - rename $oldname to $newname");
-      writehistory("rename folder - rename $oldname to $newname");
+      writelog("rename folder - $oldname to $newname");
+      writehistory("rename folder - $oldname to $newname");
    }
 
-#   print "Location: $config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&page=$page\n\n";
+#   print redirect(-location=>"$config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&page=$page");
    getfolders(\@validfolders, \$folderusage);
    editfolders();
 }
@@ -627,13 +655,14 @@ sub downloadfolder {
    $filename=~s/\s+/_/g;
 
    filelock($folderfile, LOCK_EX|LOCK_NB) or
-      openwebmailerror("$lang_err{'couldnt_lock'} $folderfile");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderfile");
 
    # disposition:attachment default to save
-   print qq|Content-Transfer-Coding: binary\n|,
-         qq|Connection: close\n|,
+   print qq|Connection: close\n|,
          qq|Content-Type: $contenttype; name="$filename"\n|;
-   if ( $ENV{'HTTP_USER_AGENT'}!~/MSIE 5.5/ ) {	# ie5.5 is broken with content-disposition
+   if ( $ENV{'HTTP_USER_AGENT'}=~/MSIE 5.5/ ) {	# ie5.5 is broken with content-disposition: attachment
+      print qq|Content-Disposition: filename="$filename"\n|;
+   } else {
       print qq|Content-Disposition: attachment; filename="$filename"\n|;
    }
    print qq|\n|;
@@ -653,4 +682,3 @@ sub downloadfolder {
    return;
 }
 ################## END DOWNLOADFOLDER #####################
-
