@@ -17,7 +17,6 @@ $ENV{BASH_ENV} = ""; # no startup script for bash
 umask(0002); # make sure the openwebmail group can write
 
 use strict;
-use IPC::Open3;
 use Fcntl qw(:DEFAULT :flock);
 use CGI qw(-private_tempfiles :standard);
 use CGI::Carp qw(fatalsToBrowser carpout);
@@ -27,30 +26,39 @@ require "filelock.pl";
 require "mime.pl";
 require "iconv.pl";
 require "maildb.pl";
+require "htmlrender.pl";
+require "htmltext.pl";
 
+# common globals
 use vars qw(%config %config_raw);
 use vars qw($thissession);
 use vars qw($domain $user $userrealname $uuid $ugid $homedir);
 use vars qw(%prefs %style);
+use vars qw($quotausage $quotalimit);
 use vars qw($folderdir @validfolders $folderusage);
 use vars qw($folder $printfolder $escapedfolder);
 
-openwebmail_init();
+# extern vars
+use vars qw(%lang_text %lang_err);	# defined in lang/xy
+use vars qw($_SUBJECT $_CHARSET);	# defined in maildb.pl
 
+# local global
 use vars qw($sort $page);
 use vars qw($searchtype $keyword $escapedkeyword);
 
+########################## MAIN ##############################
+openwebmail_requestbegin();
+$SIG{PIPE}=\&openwebmail_exit;	# for user stop
+$SIG{TERM}=\&openwebmail_exit;	# for user stop
+
+userenv_init();
+
 $page = param("page") || 1;
 $sort = param("sort") || $prefs{'sort'} || 'date';
 $keyword = param("keyword") || '';
 $escapedkeyword = escapeURL($keyword);
 $searchtype = param("searchtype") || 'subject';
 
-# extern vars
-use vars qw(%lang_text %lang_err);	# defined in lang/xy
-use vars qw($_SUBJECT $_CHARSET);	# defined in maildb.pl
-
-########################## MAIN ##############################
 my $action = param("action");
 if ($action eq "viewattachment") {
    viewattachment();
@@ -61,11 +69,10 @@ if ($action eq "viewattachment") {
 } elsif ($action eq "saveattfile" && $config{'enable_webdisk'}) {
    saveattfile();
 } else {
-   openwebmailerror("Action $lang_err{'has_illegal_chars'}");
+   openwebmailerror(__FILE__, __LINE__, "Action $lang_err{'has_illegal_chars'}");
 }
 
-# back to root if possible, required for setuid under persistent perl
-$<=0; $>=0;
+openwebmail_requestend();
 ###################### END MAIN ##############################
 
 ################ VIEWATTACHMENT/SAVEATTACHMENT ##################
@@ -74,7 +81,22 @@ sub viewattachment {	# view attachments inside a message
    my $nodeid = param("attachment_nodeid");
 
    my ($attfilename, $length, $r_attheader, $r_attbody)=getattachment($folder, $messageid, $nodeid);
-   print "${$r_attheader}\n${$r_attbody}";
+
+   if (${$r_attheader}=~m!Content-Type: text/!i && $length>512 &&
+       cookie("openwebmail-httpcompress") &&
+       $ENV{'HTTP_ACCEPT_ENCODING'}=~/\bgzip\b/ &&
+       has_zlib()) {
+      my $zattbody=Compress::Zlib::memGzip($r_attbody);
+      my $zlen=length($zattbody);
+      my $zattheader=qq|Content-Encoding: gzip\n|.
+                     qq|Vary: Accept-Encoding\n|.
+                     ${$r_attheader};
+      $zattheader=~s!Content\-Length: .*?\n!Content-Length: $zlen\n!ims;
+      print $zattheader, "\n", $zattbody;
+   } else {
+      print ${$r_attheader}, "\n", ${$r_attbody};
+   }
+   return;
 }
 
 sub saveattachment {	# save attachments inside a message to webdisk
@@ -92,10 +114,10 @@ sub getattachment {
    my $folderhandle=do { local *FH };
 
    filelock($folderfile, LOCK_SH|LOCK_NB) or
-      openwebmailerror("$lang_err{'couldnt_locksh'} $folderfile!");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $folderfile!");
    if (update_headerdb($headerdb, $folderfile)<0) {
       filelock($folderfile, LOCK_UN);
-      openwebmailerror("$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
    }
    open($folderhandle, "$folderfile");
    my $r_block= get_message_block($messageid, $headerdb, $folderhandle);
@@ -103,7 +125,7 @@ sub getattachment {
    filelock($folderfile, LOCK_UN);
 
    if ( !defined(${$r_block}) ) {
-      openwebmailerror("What the heck? Message ".str2html($messageid)." seems to be gone!");
+      openwebmailerror(__FILE__, __LINE__, "What the heck? Message ".str2html($messageid)." seems to be gone!");
    }
 
    my @attr=get_message_attributes($messageid, $headerdb);
@@ -125,14 +147,23 @@ sub getattachment {
       $subject =~ s/\s+/_/g;
 
       my $length = length(${$r_block});
-      # disposition:attachment default to save
       my $attheader=qq|Content-Length: $length\n|.
-                    qq|Content-Transfer-Coding: binary\n|.
                     qq|Connection: close\n|.
                     qq|Content-Type: message/rfc822; name="$subject.msg"\n|;
-      if ( $ENV{'HTTP_USER_AGENT'}!~/MSIE 5.5/ ) {	# ie5.5 is broken with content-disposition
+
+      # disposition:attachment default to save
+      if ( $ENV{'HTTP_USER_AGENT'}=~/MSIE 5.5/ ) {	# ie5.5 is broken with content-disposition: attachment
+         $attheader.=qq|Content-Disposition: filename="$subject.msg"\n|;
+      } else {
          $attheader.=qq|Content-Disposition: attachment; filename="$subject.msg"\n|;
       }
+
+      # allow cache for msg in folder other than saved-drafts
+      if ($folder ne 'saved-drafts') {
+         $attheader.=qq|Expires: |.CGI::expires('+900s').qq|\n|.
+                     qq|Cache-Control: private,max-age=900\n|;
+      }
+
       return("$subject.msg", $length, \$attheader, $r_block);
 
    } else {
@@ -171,8 +202,8 @@ sub getattachment {
             my $escapedmessageid = escapeURL($messageid);
             $content = html4nobase($content);
 #            $content = html4link($content);
-            $content = html4disablejs($content) if ($prefs{'disablejs'}==1);
-            $content = html4disableembcgi($content) if ($prefs{'disableembcgi'}==1);
+            $content = html4disablejs($content) if ($prefs{'disablejs'});
+            $content = html4disableemblink($content, $prefs{'disableemblink'}) if ($prefs{'disableemblink'} ne 'none');
             $content = html4attachments($content, $r_attachments, "$config{'ow_cgiurl'}/openwebmail-viewatt.pl", "action=viewattachment&amp;sessionid=$thissession&amp;message_id=$escapedmessageid&amp;folder=$escapedfolder");
 #            $content = html4mailto($content, "$config{'ow_cgiurl'}/openwebmail-send.pl", "action=composemessage&amp;sort=$sort&amp;keyword=$escapedkeyword&amp;searchtype=$searchtype&amp;folder=$escapedfolder&amp;page=$page&amp;sessionid=$thissession&amp;composetype=sendto");
          }
@@ -201,7 +232,7 @@ sub getattachment {
          # from *.exe, *.com *.bat, *.pif, *.lnk, *.scr to *.file
          # if its contenttype is not application/octet-stream
          # to avoid this attachment is referenced by html and executed directly ie
-         if ( $filename =~ /\.(exe|com|bat|pif|lnk|scr)$/i &&
+         if ( $filename =~ /\.(?:exe|com|bat|pif|lnk|scr)$/i &&
               $contenttype !~ /application\/octet\-stream/i &&
               $contenttype !~ /application\/x\-msdownload/i ) {
             $filename="$filename.file";
@@ -213,27 +244,35 @@ sub getattachment {
             $contenttype="image/".lc($1);
          }
 
-         # disposition:attachment default to save
          my $attheader=qq|Content-Length: $length\n|.
-                       qq|Content-Transfer-Coding: binary\n|.
                        qq|Connection: close\n|.
                        qq|Content-Type: $contenttype; name="$filename"\n|;
-         if ( $ENV{'HTTP_USER_AGENT'}!~/MSIE 5.5/ ) {	# ie5.5 is broken with content-disposition
-            if ($contenttype =~ /^text/i) {
-               $attheader.=qq|Content-Disposition: inline; filename="$filename"\n|;
+         if ($contenttype =~ /^text/i) {
+            $attheader.=qq|Content-Disposition: inline; filename="$filename"\n|;
+         } else {
+            # disposition:attachment default to save
+            if ( $ENV{'HTTP_USER_AGENT'}=~/MSIE 5.5/ ) { # ie5.5 is broken with content-disposition: attachment
+               $attheader.=qq|Content-Disposition: filename="$filename"\n|;
             } else {
                $attheader.=qq|Content-Disposition: attachment; filename="$filename"\n|;
             }
          }
 
+         # allow cache for msg attachment in folder other than saved-drafts
+         if ($folder ne 'saved-drafts') {
+            $attheader.=qq|Expires: |.CGI::expires('+900s').qq|\n|.
+                        qq|Cache-Control: private,max-age=900\n|;
+         }
+
          # use undef to free memory before attachment transfer
          undef %{$r_attachment};
          undef $r_attachment;
          undef @{$r_attachments};
          undef $r_attachments;
+
          return($filename, $length, \$attheader, \$content);
       } else {
-         openwebmailerror("What the heck? Message ".str2html($messageid)." $nodeid seems to be gone!");
+         openwebmailerror(__FILE__, __LINE__, "What the heck? Message ".str2html($messageid)." $nodeid seems to be gone!");
       }
    }
    # never reach
@@ -242,11 +281,24 @@ sub getattachment {
 
 ################ VIEWATTFILE/SAVEATTFILE ##################
 sub viewattfile {	# view attachments uploaded to $config{'ow_sessionsdir'}
-   my $attfile=param("attfile");
-   $attfile =~ s/\///g;  # just in case someone gets tricky ...
-
+   my $attfile=param("attfile"); $attfile =~ s/\///g;  # just in case someone gets tricky ...
    my ($attfilename, $length, $r_attheader, $r_attbody)=getattfile($attfile);
-   print "${$r_attheader}\n${$r_attbody}";
+
+   if (${$r_attheader}=~m!Content-Type: text/!i && $length>512 &&
+       cookie("openwebmail-httpcompress") &&
+       $ENV{'HTTP_ACCEPT_ENCODING'}=~/\bgzip\b/ &&
+       has_zlib()) {
+      my $zattbody=Compress::Zlib::memGzip($r_attbody);
+      my $zlen=length($zattbody);
+      my $zattheader=qq|Content-Encoding: gzip\n|.
+                     qq|Vary: Accept-Encoding\n|.
+                     ${$r_attheader};
+      $zattheader=~s!Content\-Length: .*?\n!Content-Length: $zlen\n!ims;
+      print $zattheader, "\n", $zattbody;
+   } else {
+      print ${$r_attheader}, "\n", ${$r_attbody};
+   }
+   return;
 }
 
 sub saveattfile {	# save attachments uploaded to $config{'pw_sessiondir'} to webdisk
@@ -260,8 +312,8 @@ sub saveattfile {	# save attachments uploaded to $config{'pw_sessiondir'} to web
 sub getattfile {
    my $attfile=$_[0];
    # only allow to view attfiles belongs the $thissession
-   if ($attfile!~/^$thissession/  || !-f "$config{'ow_sessionsdir'}/$attfile") {
-      openwebmailerror("What the heck? Attfile $config{'ow_sessionsdir'}/$attfile seems to be gone!");
+   if ($attfile!~/^\Q$thissession\E/  || !-f "$config{'ow_sessionsdir'}/$attfile") {
+      openwebmailerror(__FILE__, __LINE__, "What the heck? Attfile $config{'ow_sessionsdir'}/$attfile seems to be gone!");
    }
 
    my ($attsize, $attheader, $attheaderlen, $attcontent);
@@ -271,7 +323,7 @@ sub getattfile {
    $attsize=(-s("$config{'ow_sessionsdir'}/$attfile"));
 
    open(ATTFILE, "$config{'ow_sessionsdir'}/$attfile") or
-      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_sessionsdir'}/$attfile!");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $config{'ow_sessionsdir'}/$attfile! ($!)");
    read(ATTFILE, $attheader, 512);
    $attheaderlen=index($attheader,  "\n\n", 0);
    $attheader=substr($attheader, 0, $attheaderlen);
@@ -337,11 +389,14 @@ sub getattfile {
    # rebuild attheader for download
    # disposition:inline default to open
    $attheader= qq|Content-Length: $length\n|.
-               qq|Content-Transfer-Coding: binary\n|.
                qq|Connection: close\n|.
                qq|Content-Type: $attcontenttype; name="$attfilename"\n|.
                qq|Content-Disposition: inline; filename="$attfilename"\n|;
 
+   # allow cache for attfile since its filename is based on times()
+   $attheader.=qq|Expires: |.CGI::expires('+900s').qq|\n|.
+               qq|Cache-Control: private,max-age=900\n|;
+
    return($attfilename, $length, \$attheader, \$attcontent);
 }
 ################### END VIEWATTATTFILE ##################
@@ -349,26 +404,24 @@ sub getattfile {
 ##################### SAVEFILE2WEBDISK ###################
 sub savefile2webdisk {
    my ($filename, $length, $r_content, $webdisksel)=@_;
-   my $webdiskrootdir=$homedir.absolute_vpath("/", $config{'webdisk_rootpath'});
-   ($webdiskrootdir =~ m!^(.+)/?$!) && ($webdiskrootdir = $1);  # untaint ...
 
-   if ($config{'webdisk_quota'}>0) {
-      my $webdiskusage=0;
-      my ($stdout, $stderr, $exit, $sig)=execute('/usr/bin/du', '-sk', $webdiskrootdir);
-      $webdiskusage=$1 if ($stdout=~/(\d+)/);
-      if ($length/1024+$webdiskusage > $config{'webdisk_quota'}) {
-         autoclosewindow($lang_text{'quota_hit'}, $lang_err{'webdisk_hitquota'});
+   if ($quotalimit>0 && $quotausage+$length/1024>$quotalimit) {
+      $quotausage=(quota_get_usage_limit(\%config, $user, $homedir, 1))[2];	# get uptodate quotausage
+      if ($quotausage + $length/1024 > $quotalimit) {
+         autoclosewindow($lang_text{'quotahit'}, $lang_err{'quotahit_alert'});
       }
    }
 
+   my $webdiskrootdir=$homedir.absolute_vpath("/", $config{'webdisk_rootpath'});
+   ($webdiskrootdir =~ m!^(.+)/?$!) && ($webdiskrootdir = $1);  # untaint ...
    my $vpath=absolute_vpath('/', $webdisksel);
    my $err=verify_vpath($webdiskrootdir, $vpath);
-   openwebmailerror($err) if ($err);
+   openwebmailerror(__FILE__, __LINE__, $err) if ($err);
 
    if (-d "$webdiskrootdir/$vpath") {			# use choose a dirname, save att with its original name
       $vpath=absolute_vpath($vpath, $filename);
       $err=verify_vpath($webdiskrootdir, $vpath);
-      openwebmailerror($err) if ($err);
+      openwebmailerror(__FILE__, __LINE__, $err) if ($err);
    }
    ($vpath =~ /^(.+)$/) && ($vpath = $1);  # untaint ...
 
@@ -382,8 +435,8 @@ sub savefile2webdisk {
    chmod(0644, "$webdiskrootdir/$vpath");
    filelock("$webdiskrootdir/$vpath", LOCK_UN);
 
-   writelog("saveatt - $vpath");
-   writehistory("saveatt - $vpath");
+   writelog("save attachment - $vpath");
+   writehistory("save attachment - $vpath");
 
    autoclosewindow($lang_text{'savefile'}, "$lang_text{'savefile'} $lang_text{'succeeded'} ($vpath)");
 }
