@@ -6,10 +6,6 @@ use strict;
 no strict 'vars';
 push (@INC, '/usr/local/www/cgi-bin/openwebmail', ".");
 
-sub log_time {
-   print(join(" ", @_), "\n");
-}
-
 require "auth_pam.pl";
 
 my ($user, $passwd, $newpasswd)=@ARGV;
@@ -23,12 +19,12 @@ $<=$>;
 print "user=$user, pass=$passwd, newpass=$newpasswd\n";
 
 print "check_userpassword ret=",
-      check_userpassword($user, $passwd), 
+      openwebmail::auth_pam::check_userpassword($user, $passwd), 
       "\n";
 
 print "\n\n";
 
 print "change_userpassword ret=",
-      change_userpassword($user, $passwd, $newpasswd), 
+      openwebmail::auth_pam::change_userpassword($user, $passwd, $newpasswd), 
       "\n";
 
